#ifndef MPIB_H_
#define MPIB_H_

/*!
 * \mainpage Introduction 
Accurate estimation of the execution time of MPI communication operations plays an important role in optimization of parallel applications. A priori information about the performance of each MPI operation allows a software developer to design a parallel application in such a way that it will have maximum performance. This data can also be useful for tuning collective communication operations and for the evaluation of different available implementations. The choice of collective algorithms becomes even more important in heterogeneous environments.

A typical MPI benchmarking suite uses only one timing method to estimate the execution time of the MPI communications. The method provides a certain accuracy and efficiency. The efficiency of the timing method is particularly important in self-adaptable parallel applications using runtime benchmarking of communication operations to optimize their performance on the executing platform. In this case, less accurate results can be acceptable in favor of a rapid response from the benchmark. We design a new MPI benchmarking suite called MPIBlib that provides a variety of timing methods. This suite supports both fast measurement of collective operations and exhaustive benchmarking.

In addition to general timing methods that are universally applicable to all communication operations, MPIBlib includes methods that can only be used for measurement of one or more specific operations. Where applicable, these operation-specific methods work faster than their universal counterparts and can be used as their time-efficient alternatives.

Most of the MPI benchmarking suites are designed in the form of a standalone executable program that takes the parameters of communication experiments and produce a lot of output data for further analysis. As such, they cannot be integrated easily and efficiently into application-level software. Therefore, there is a need for a benchmarking library that can be used in parallel applications or programming systems for communication performance modeling and tuning communication operations. MPIBlib is such a library that can be linked to other applications and used at runtime. 
 * 
 * \section authors Authors
 * \include AUTHORS
 * 
 * \section Changes
 * \include NEWS
 * 
 * \page installation Installation
 * \include README
 * 
 * \page usage Usage
 * The package consists of a library, executables and gnuplot scripts.
 *
 * Benchmarking executables and gnuplot scripts that visualize the results are described in:
 * - \ref collective/main.c
 * - \ref p2p/main.c
 * 
 * Typical parameters of executables are as follows:
 * - \b -h help
 * - \b -O \e S collective operation (required):\n
			MPI_Scatter, MPIB_Scatter_linear, MPIB_Scatter_binomial\n
			MPI_Gather, MPIB_Gather_linear, MPIB_Gather_binomial\n
			MPI_Scatterv\n
			MPI_Gatherv\n
 * - \b -t \e S timing: max, root, global (default: max)
 * - \b -s \e I message size stride (default: 1024)
 * - \b -m \e I maximum message size (default: 102400)
 * - \b -p parallel p2p benchmarking (default: 1)
 * - \b -r \e I minimum number of repetitions (default: 5)
 * - \b -R \e I maximum number of repetitions (default: 100)
 * - \b -c \e D confidence level: 0 < D < 1 (default: 0.95)
 * - \b -e \e D error: 0 < D < 1 (default: 0.025)
 * 
 * where:
 * - \e S - string
 * - \e I - integer
 * - \e D - double
 * 
 * Using the gnuplot
 * \code
 * $ gnuplot script_name.plot
 * \endcode
 * The gnuplot data files should have names script_name.out
 */

#include <mpi.h>

#include "mpib_hetero.h"
#include "mpib_collective.h"
#include "mpib_container.h"
#include "mpib_measurement.h"
#include "mpib_utilities.h"

#endif /*MPIB_H_*/
