#include "mpib.h"
#include <malloc.h>
#include <memory.h>

void MPIB_test_scatter(MPIB_Scatter scatter, MPI_Comm comm, int root, int* res)
{
	int rank;
	MPI_Comm_rank(comm, &rank);
	int size;
	MPI_Comm_size(comm, &size);
	int M = 1024;
	int* send_buffer = NULL;
	if (rank == root)
	{
		send_buffer = (int*)malloc(sizeof(int) * M);
		int i;
		for (i = 0; i < M; i++)
			send_buffer[i] = i;
	}
	int m = M / size;
	int* recv_buffer = (int*)calloc(m, sizeof(int));
	scatter(send_buffer, m, MPI_INT, recv_buffer, m, MPI_INT, root, comm);
	int flag = 0;
	int i;
	for (i = 0; i < m; i++)
		flag |= recv_buffer[i] != i + rank * m;
	MPI_Allreduce(&flag, res, 1, MPI_INT, MPI_MAX, comm);
	free(send_buffer);
	free(recv_buffer);
}

void MPIB_test_gather(MPIB_Gather gather, MPI_Comm comm, int root, int* res)
{
	int rank;
	MPI_Comm_rank(comm, &rank);
	int size;
	MPI_Comm_size(comm, &size);
	int M = 1024;
	int* recv_buffer = NULL;
	if (rank == root)
		recv_buffer = (int*)calloc(M, sizeof(int));
	int m = M / size;
	int* send_buffer = (int*)malloc(sizeof(int) * m);
	int i;
	for (i = 0; i < m; i++)
		send_buffer[i] = i + rank * m;
	gather(send_buffer, m, MPI_INT, recv_buffer, m, MPI_INT, root, comm);
	if (rank == root)
	{
		*res = 0;
		int i, s;
		for (i = 0, s = m * size; i < s; i++)
			*res |= recv_buffer[i] != i;
	}
	MPI_Bcast(res, 1, MPI_INT, root, comm);
	free(send_buffer);
	free(recv_buffer);
}
