#include "mpib.h"
#include <memory.h>

int MPIB_Scatter_linear(void* sendbuf, int sendcount, MPI_Datatype sendtype, 
	void* recvbuf, int recvcount, MPI_Datatype recvtype, 
	int root, MPI_Comm comm)
{
	int rank;
	MPI_Comm_rank(comm, &rank);
	if (rank == root)
	{
		int err = MPI_SUCCESS;
		int size;
		MPI_Comm_size(comm, &size);
		MPI_Aint extent;
		MPI_Type_extent(sendtype, &extent);
		int inc = sendcount * extent;
		int i;
		void* ptr;
		for (i = 0, ptr = sendbuf; i < size; i++, ptr += inc)
		{
			if (i == root)
			{
				MPI_Aint extent;
				MPI_Type_extent(recvtype, &extent);
				int inc = recvcount * extent;
				memcpy(recvbuf, ptr, inc);
			}
			else
				err &= MPIB_Send(ptr, sendcount, sendtype, i, 0, comm);
		}
		return err;
	}
	else
		return MPIB_Recv(recvbuf, recvcount, recvtype, root, 0, comm, MPI_STATUS_IGNORE);
}

int MPIB_Gather_linear(void* sendbuf, int sendcount, MPI_Datatype sendtype, 
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	int rank;
	MPI_Comm_rank(comm, &rank);
	if (rank == root)
	{
		int err = MPI_SUCCESS;
		int size;
		MPI_Comm_size(comm, &size);
		MPI_Aint extent;
		MPI_Type_extent(recvtype, &extent);
		int inc = recvcount * extent;
		int i;
		int c;
		void* ptr;
		for (i = 0, c = 0, ptr = recvbuf; i < size; i++, ptr += inc)
		{
			if (i == root)
				memcpy(ptr, sendbuf, inc);
			else
				err &= MPIB_Recv(ptr, recvcount, recvtype, i, 0, comm, MPI_STATUS_IGNORE);
		}
		return err;
	}
	else
		return MPIB_Send(sendbuf, sendcount, sendtype, root, 0, comm);
}
