#include "mpib.h"
#include <malloc.h>
#include <unistd.h>

/*! \brief Parameters of emulated heterogeneity of processors */
typedef struct MPIB_hetero_procs
{
	/*! \brief MPI communicator */
	MPI_Comm comm;
	/*! \brief array of fixed delays (microsec) */
	int* a;
	/*! \brief array of variable delays (microsec) */
	int* b;
}
MPIB_hetero_procs;

/*! \brief Parameters of emulated heterogeneity of processors */
static MPIB_hetero_procs* MPIB_hetero_procs_instance = NULL;

void MPIB_hetero_procs_init(MPI_Comm comm)
{
	int size;
	MPI_Comm_size(comm, &size);
	MPIB_hetero_procs_instance = (MPIB_hetero_procs*)malloc(sizeof(MPIB_hetero_procs));
	MPIB_hetero_procs_instance->a = (int*)malloc(sizeof(int) * size);
	MPIB_hetero_procs_instance->b = (int*)malloc(sizeof(int) * size);
	int i;
	for (i = 0; i < size; i++)
	{
		MPIB_hetero_procs_instance->a[i] = 100 * i;
		MPIB_hetero_procs_instance->b[i] = 100 * i;
	}
}

void MPIB_hetero_procs_free()
{
	if (MPIB_hetero_procs_instance)
	{
		free(MPIB_hetero_procs_instance->a);
		free(MPIB_hetero_procs_instance->b);
		free(MPIB_hetero_procs_instance);
	}
}

int MPIB_Send(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm)
{
	if (MPIB_hetero_procs_instance && MPIB_hetero_procs_instance->comm == comm)
	{
		int rank;
		MPI_Comm_rank(comm, &rank);
		MPI_Aint extent;
		MPI_Type_extent(datatype, &extent);
		usleep(MPIB_hetero_procs_instance->a[rank] + MPIB_hetero_procs_instance->b[rank] * extent);
	}
	return MPI_Send(buf, count, datatype, dest, tag, comm);
}

int MPIB_Recv(void* buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status* status)
{
	if (MPIB_hetero_procs_instance && MPIB_hetero_procs_instance->comm == comm)
	{
		int rank;
		MPI_Comm_rank(comm, &rank);
		MPI_Aint extent;
		MPI_Type_extent(datatype, &extent);
		usleep(MPIB_hetero_procs_instance->a[rank] + MPIB_hetero_procs_instance->b[rank] * extent);
	}
	return MPI_Recv(buf, count, datatype, source, tag, comm, status);
}
