#ifndef MPIB_HETERO_H_
#define MPIB_HETERO_H_

/*!
 * \defgroup hetero Emulated heterogeneity
 * \{
 */

/*! \brief Initializes the parameters of emulated heterogeneity of processors. */
void MPIB_hetero_procs_init(MPI_Comm comm);

/*! \brief Frees the parameters of emulated heterogeneity of processors. */
void MPIB_hetero_procs_free();

/*!
 * \brief Slow MPI_Send based on emulated heterogeneity of processors
 * \details Requires \ref MPIB_hetero_procs_init
 */
int MPIB_Send(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm); 

/*!
 * \brief Slow MPI_Recv based on emulated heterogeneity of processors
 * \details Requires \ref MPIB_hetero_procs_init
 */
int MPIB_Recv(void* buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status* status); 

/*!
 * \}
 */

#endif /*MPIB_HETERO_H_*/
