#include "mpib.h"
#include <malloc.h>

void MPIB_measure_max(MPIB_coll_container* container, MPI_Comm comm, int root, int M,
	MPIB_precision precision, MPIB_result* result)
{
	double* T = (double*)malloc(sizeof(double) * precision.max_reps);

	container->initialize(container, comm, root, M);
	int stop = 0;
	double sum = 0;
	int reps = 0;
	double ci = 0;
	while (!stop && reps < precision.max_reps)
	{
		MPI_Barrier(comm); MPI_Barrier(comm);
		double time = MPI_Wtime();
		container->execute(container, comm, root, M);
		time = MPI_Wtime() - time;
		MPI_Allreduce(&time, &T[reps], 1, MPI_DOUBLE, MPI_MAX, comm);
		sum += T[reps];
		reps++;
		if (reps >= precision.min_reps && reps > 2)
			stop = (ci = MPIB_ci(precision.cl, reps, T)) * reps / sum < precision.eps;
	}
	container->finalize(container, comm, root);

	result->T = sum / reps;
	MPIB_max_wtick(comm, &result->wtick);
	result->reps = reps;
	result->ci = ci;
	free(T);
}
