#ifndef CPM_H_
#define CPM_H_

/*!
 * \mainpage Introduction 
 * CPM (Communication Performance Model) provides the modeling of performance of MPI communications.
 * 
 * \section authors Authors
 * \include AUTHORS
 * 
 * \page installation Installation
 * \include README
 * 
 * \page usage Usage
 * The package consists of a library, executables and gnuplot scripts.
 *
 * \section tools Executables and gnuplot scripts
 * Benchmarking executables and gnuplot scripts that visualize the results are described in:
 * - \ref model/main.c
 * - \ref collective/main.c
 * - \ref p2p/main.c
 * 
 * Typical parameters of executables are as follows:
 * - \b -h help
 * - \b -i \e S	input model file, stdin - standard input (default: none - model will be built)
 * - \b -o \e S	output model file, stdout - standard output (default: stdout)
 * - \b -O \e S collective operation (required):\n
			MPI_Scatter, MPIB_Scatter_linear, MPIB_Scatter_binomial\n
			MPI_Gather, MPIB_Gather_linear, MPIB_Gather_binomial\n
			MPI_Scatterv\n
			MPI_Gatherv\n
			CPM_Scatter_opt\n
			CPM_Gather_opt\n
			CPM_Scatter_binomial_opt, CPM_Scatter_binomial_cor\n
			CPM_Gather_binomial_opt, CPM_Gather_binomial_cor\n
 * - \b -t \e S timing: max, root, global (default: max)
 * - \b -s \e I message size stride (default: 1024)
 * - \b -m \e I maximum message size (default: 102400)
 * - \b -p parallel p2p benchmarking (default: 1)
 * - \b -r \e I minimum number of repetitions (default: 5)
 * - \b -R \e I maximum number of repetitions (default: 100)
 * - \b -c \e D confidence level: 0 < D < 1 (default: 0.95)
 * - \b -e \e D error: 0 < D < 1 (default: 0.025)
 * 
 * where:
 * - \e S - string
 * - \e I - integer
 * - \e D - double
 * 
 * Using the gnuplot
 * \code
 * $ gnuplot script_name.plot
 * \endcode
 * The gnuplot data files should have names script_name.out
 */

#include "mpib.h"

#include "cpm_config.h"

#include "cpm_utilities.h"
#include "cpm_measurement.h"
#include "cpm_model.h"
#include "cpm_hockney.h"
#ifdef HAVE_LIBLOGP
#include "cpm_plogp.h"
#endif /*HAVE_LIBLOGP*/
#include "cpm_collective.h"

/*!
 * \defgroup executables Executables
*/

#endif /*CPM_H_*/
