#ifndef CPM_UTILITIES_H_
#define CPM_UTILITIES_H_

/*!
 * \defgroup Utilities Utilities
 * \brief Utility functions.
 * \{
 */

/*!
 * \brief \f$ C_n^3 \f$
 */
#define CPM_C3(n) (n) * ((n) - 1) * ((n) - 2) / 6

/*!
 * \brief The index of the \f$ (i, j, k) \f$ element in the array of \f$ C_n^3 \f$ elements, \f$ i < j < k < n \f$
 * \details \f$ \frac{C_{n - 1}^2 + C_{n - i}^2}{2} i + \frac{(n - i + 1) + (n - j)}{2} (j - i) + (k - j - 1) \f$
 */
#define CPM_IJK2INDEX(n, i, j, k) (2 * (n) - (i) - 1) * ((i) - 1) * (i) / 4 + (2 * (n) - (i) - (j) + 1) * ((j) - (i)) / 2 + (k) - (j) - 1

void CPM_print_p2pp_results(int n, int M, MPIB_precision precision, MPIB_result* p2pp, MPIB_result* p2p_0, MPIB_result* p2p_M);

void CPM_print_p2pp_results_(int n, int M, MPIB_precision precision, MPIB_result* p2pp, MPIB_result* p2p_0[2], MPIB_result* p2p_M[2]);

/*!
 * \}
 */

#endif /*CPM_UTILITIES_H_*/
